
# Functions to support drawing paths imported by 'grImport' package

# Differs from grobify() in grImport because it
# fills PictureFill and PictureChar paths as paths with holes
setGeneric("svgGrobify",
           function(object, ...) {
               standardGeneric("svgGrobify")
           })

setMethod("svgGrobify", signature(object="ANY"),
          function(object, ...) {
              grImport::grobify(object, FUN=svgGrobify, ...)
          })

setMethod("svgGrobify", signature(object="PictureFill"),
          function(object, ..., fillText, bgText, sizeByWidth, use.gc=TRUE) {
              if (length(object@x) > 1) {
                  paths <- grImport:::explode(object)
                  if (is.list(paths)) {
                      pathX <- lapply(paths, slot, "x")
                      pathY <- lapply(paths, slot, "y")
                  } else {
                      pathX <- list(paths@x)
                      pathY <- list(paths@y)
                  }
                  if (use.gc) {
                      pathGrob(x=unlist(pathX),
                               y=unlist(pathY),
                               default.units="native",
                               id.lengths=sapply(pathX, length),
                               gp=gpar(col=NA, fill=object@rgb,
                                 fillrule=object@rule),
                               ...)
                  } else {
                      pathGrob(x=unlist(pathX),
                               y=unlist(pathY),
                               default.units="native",
                               id.lengths=sapply(pathX, length),
                               ...)
                  }
              } else {
                  NULL
              }
          })

setMethod("svgGrobify", signature(object="PictureChar"),
          function(object, ...,
                   fillText=FALSE, bgText=NA,
                   sizeByWidth=TRUE, use.gc=TRUE) {
              paths <- grImport:::explode(object, FALSE, NA)
              pathX <- lapply(paths, slot, "x")
              pathY <- lapply(paths, slot, "y")
              if (use.gc) {
                  pathGrob(x=unlist(pathX),
                           y=unlist(pathY),
                           default.units="native",
                           id.lengths=sapply(pathX, length),
                           gp=gpar(col=NA, fill=object@rgb,
                             fillrule="nonzero"),
                           ...)
              } else {
                  pathGrob(x=unlist(pathX),
                           y=unlist(pathY),
                           default.units="native",
                           id.lengths=sapply(pathX, length),
                           ...)
              }
          })


