
# A grid primitive to draw a path
# The idea is to allow path with sub-path to create holes
# (e.g., letters!)

validDetails.pathgrob <- function(x) {
  if (!is.unit(x$x) ||
      !is.unit(x$y))
    stop("'x' and 'y' must be units")
  if (!is.null(x$id) && !is.null(x$id.lengths))
    stop("It is invalid to specify both 'id' and 'id.lengths'")
  if (length(x$x) != length(x$y))
    stop("'x' and 'y' must be same length")
  if (!is.null(x$id) && (length(x$id) != length(x$x)))
    stop("'x' and 'y' and 'id' must all be same length")
  if (!is.null(x$id))
    x$id <- as.integer(x$id)
  if (!is.null(x$id.lengths) && (sum(x$id.lengths) != length(x$x)))
    stop("'x' and 'y' and 'id.lengths' must specify same overall length")
  if (!is.null(x$id.lengths))
    x$id.lengths <- as.integer(x$id.lengths)
  x
}

# A path draws as a polygon in normal graphics
# Will NOT look right because holes will NOT be filled
# but at least shows where everything goes
drawDetails.pathgrob <- function(x, recording=TRUE) {
    if (is.null(x$id) && is.null(x$id.lengths))
        grid:::grid.Call.graphics("L_polygon", x$x, x$y,
                                  list(as.integer(seq_along(x$x))))
    else {
        if (is.null(x$id)) {
            n <- length(x$id.lengths)
            id <- rep(1L:n, x$id.lengths)
        } else {
            n <- length(unique(x$id))
            id <- x$id
        }
        index <- split(as.integer(seq_along(x$x)), id)
        grid:::grid.Call.graphics("L_polygon", x$x, x$y, index)
    }
}

xDetails.pathgrob <- function(x, theta) {
    bounds <- grid:::grid.Call("L_locnBounds", x$x, x$y, theta)
    if (is.null(bounds))
        unit(0.5, "npc")
    else
        unit(bounds[1L], "inches")
}

yDetails.pathgrob <- function(x, theta) {
    bounds <- grid:::grid.Call("L_locnBounds", x$x, x$y, theta)
    if (is.null(bounds))
        unit(0.5, "npc")
    else
        unit(bounds[2L], "inches")
}

widthDetails.pathgrob <- function(x) {
  bounds <- grid:::grid.Call("L_locnBounds", x$x, x$y, 0)
  if (is.null(bounds))
    unit(0, "inches")
  else
    unit(bounds[3L], "inches")
}

heightDetails.pathgrob <- function(x) {
  bounds <- grid:::grid.Call("L_locnBounds", x$x, x$y, 0)
  if (is.null(bounds))
    unit(0, "inches")
  else
    unit(bounds[4L], "inches")
}

pathGrob <- function(x=c(0, 0.5, 1, 0.5), y=c(0.5, 1, 0.5, 0),
                     id=NULL, id.lengths=NULL,
                     default.units="npc",
                     name=NULL, gp=gpar(), vp=NULL) {
  if (!is.unit(x))
    x <- unit(x, default.units)
  if (!is.unit(y))
    y <- unit(y, default.units)
  grob(x=x, y=y, id=id,
       id.lengths=id.lengths, 
       name=name, gp=gp, vp=vp, cl="pathgrob")
}

grid.path <- function(x=c(0, 0.5, 1, 0.5), y=c(0.5, 1, 0.5, 0),
                      id=NULL, id.lengths=NULL,
                      default.units="npc",
                      name=NULL, gp=gpar(), vp=NULL) {
  pg <- pathGrob(x=x, y=y, id=id, id.lengths=id.lengths,
                 default.units=default.units, 
                 name=name, gp=gp, vp=vp)
  grid.draw(pg)
  invisible(pg)
}

